@@ -39,10 +39,13 @@
 #include <htable.h>
 #include <vstring.h>
 #include <stringops.h>
+#include <argv.h>
+#include <mymalloc.h>
 
 /* Global library. */
 
-#include "header_opts.h"
+#include <mail_params.h>
+#include <header_opts.h>
 
  /*
   * Header names are given in the preferred capitalization. The lookups are
@@ -52,14 +55,14 @@
   * that enters via a non-SMTP channel. Changing this now could break other
   * software. See also comments in bounce_notify_util.c.
   */
-static const HEADER_OPTS header_opts[] = {
+static HEADER_OPTS header_opts[] = {
     "Apparently-To", HDR_APPARENTLY_TO, HDR_OPT_RECIP,
-    "Bcc", HDR_BCC, HDR_OPT_DROP | HDR_OPT_XRECIP,
+    "Bcc", HDR_BCC, HDR_OPT_XRECIP,
     "Cc", HDR_CC, HDR_OPT_XRECIP,
     "Content-Description", HDR_CONTENT_DESCRIPTION, HDR_OPT_MIME,
     "Content-Disposition", HDR_CONTENT_DISPOSITION, HDR_OPT_MIME,
     "Content-ID", HDR_CONTENT_ID, HDR_OPT_MIME,
-    "Content-Length", HDR_CONTENT_LENGTH, HDR_OPT_DROP,
+    "Content-Length", HDR_CONTENT_LENGTH, 0,
     "Content-Transfer-Encoding", HDR_CONTENT_TRANSFER_ENCODING, HDR_OPT_MIME,
     "Content-Type", HDR_CONTENT_TYPE, HDR_OPT_MIME,
     "Delivered-To", HDR_DELIVERED_TO, 0,
@@ -72,7 +75,7 @@ static const HEADER_OPTS header_opts[] = {
     "MIME-Version", HDR_MIME_VERSION, HDR_OPT_MIME,
     "Received", HDR_RECEIVED, 0,
     "Reply-To", HDR_REPLY_TO, HDR_OPT_SENDER,
-    "Resent-Bcc", HDR_RESENT_BCC, HDR_OPT_DROP | HDR_OPT_XRECIP | HDR_OPT_RR,
+    "Resent-Bcc", HDR_RESENT_BCC, HDR_OPT_XRECIP | HDR_OPT_RR,
     "Resent-Cc", HDR_RESENT_CC, HDR_OPT_XRECIP | HDR_OPT_RR,
     "Resent-Date", HDR_RESENT_DATE, HDR_OPT_RR,
     "Resent-From", HDR_RESENT_FROM, HDR_OPT_SENDER | HDR_OPT_RR,
@@ -80,7 +83,7 @@ static const HEADER_OPTS header_opts[] = {
     "Resent-Reply-To", HDR_RESENT_REPLY_TO, HDR_OPT_RECIP | HDR_OPT_RR,
     "Resent-Sender", HDR_RESENT_SENDER, HDR_OPT_SENDER | HDR_OPT_RR,
     "Resent-To", HDR_RESENT_TO, HDR_OPT_XRECIP | HDR_OPT_RR,
-    "Return-Path", HDR_RETURN_PATH, HDR_OPT_DROP | HDR_OPT_SENDER,
+    "Return-Path", HDR_RETURN_PATH, HDR_OPT_SENDER,
     "Return-Receipt-To", HDR_RETURN_RECEIPT_TO, HDR_OPT_SENDER,
     "Sender", HDR_SENDER, HDR_OPT_SENDER,
     "To", HDR_TO, HDR_OPT_XRECIP,
@@ -109,18 +112,55 @@ static void header_opts_init(void)
 	for (cp = hp->name; *cp; cp++)
 	    VSTRING_ADDCH(header_key, TOLOWER(*cp));
 	VSTRING_TERMINATE(header_key);
-	htable_enter(header_hash, vstring_str(header_key), (char *) hp);
+	htable_enter(header_hash, vstring_str(header_key), (void *) hp);
     }
 }
 
+/* header_drop_init - initialize "header drop" flags */
+
+static void header_drop_init(void)
+{
+    ARGV   *hdr_drop_list;
+    char  **cpp;
+    HTABLE_INFO *ht;
+    HEADER_OPTS *hp;
+
+    /*
+     * Having one main.cf parameter for the "drop" header flag does not
+     * generalize to the "sender", "extract", etc., flags. Flags would need
+     * to be grouped by header name, but that would be unwieldy, too:
+     * 
+     * message_header_flags = { apparently-to = recipient }, { bcc = recipient,
+     * extract, drop }, { from = sender }, ...
+     * 
+     * Thus, it is unlikely that all header flags will become configurable.
+     */
+    hdr_drop_list = argv_split(var_drop_hdrs, CHARS_COMMA_SP);
+    for (cpp = hdr_drop_list->argv; *cpp; cpp++) {
+	lowercase(*cpp);
+	if ((ht = htable_locate(header_hash, *cpp)) == 0) {
+	    hp = (HEADER_OPTS *) mymalloc(sizeof(*hp));
+	    hp->type = HDR_OTHER;
+	    hp->flags = HDR_OPT_DROP;
+	    ht = htable_enter(header_hash, *cpp, (void *) hp);
+	    hp->name = ht->key;
+	} else
+	    hp = (HEADER_OPTS *) ht->value;
+	hp->flags |= HDR_OPT_DROP;
+    }
+    argv_free(hdr_drop_list);
+}
+
 /* header_opts_find - look up header options */
 
 const HEADER_OPTS *header_opts_find(const char *string)
 {
     const char *cp;
 
-    if (header_hash == 0)
+    if (header_hash == 0) {
 	header_opts_init();
+	header_drop_init();
+    }
 
     /*
      * Look up the lower-cased version of the header name.
